\name{plots_graphics}
\alias{rug_1d_graphics}
\alias{points_1d_graphics}
\alias{jitter_1d_graphics}
\alias{density_1d_graphics}
\alias{boxplot_1d_graphics}
\alias{hist_1d_graphics}
\alias{label_1d_graphics}
\alias{arrow_1d_graphics}
\alias{rect_1d_graphics}
\alias{lines_1d_graphics}
\alias{points_2d_graphics}
\alias{density_2d_graphics}
\alias{axes_2d_graphics}
\alias{label_2d_graphics}
\alias{arrow_2d_graphics}
\alias{rect_2d_graphics}
\title{Graphics-Based Plotting Functions}
\description{
    The 1d and 2d plotting functions based on the \R package \pkg{graphics}.
}
\usage{
rug_1d_graphics(x, width=if(horizontal) 1 else 0.4,
                height=if(horizontal) 0.4 else 1, col=par("fg"),
                horizontal=TRUE, plotAsp=0.1, plotID, turn,
                add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)
points_1d_graphics(x, cex=0.4, horizontal=TRUE, plotAsp=0.1, plotID, turn,
                   add=FALSE, ann=FALSE, axes=FALSE,
                   xlim=c(0,1), ylim=c(0,1), ...)
jitter_1d_graphics(x, cex=0.4, offset=0.25, horizontal=TRUE, plotAsp=0.1, plotID, turn,
                   add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)
density_1d_graphics(x, density.args=list(), offset=0.25, method=c("single", "double"),
                    horizontal=TRUE, plotAsp=0.1, plotID, turn,
                    add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)
boxplot_1d_graphics(x, horizontal=TRUE, cex=0.4, plotAsp=0.1, range=NULL, plotID,
                    turn, add=FALSE, axes=FALSE, ...)
hist_1d_graphics(x, breaks=NULL, col=NULL, plotAsp=0.1,
                 horizontal=TRUE, plotID, turn,
                 add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1),
                 ylim=c(0,1), ...)
label_1d_graphics(loc.x=0.5, loc.y=0.5, labels=NULL,
                  srt=if(horizontal) 0 else 90, cex=0.75,
                  x, plotAsp=0.1, horizontal=TRUE, plotID, turn,
                  add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)
arrow_1d_graphics(loc=c(xlim[2], ylim[2])/2, length=0.6, angle=plotAsp*30,
                  x, horizontal=TRUE, plotAsp=0.1, plotID, turn,
                  add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)
rect_1d_graphics(loc.x=0.5, loc.y=0.5, horizontal=TRUE,
                 width=if(horizontal) 1.04 else 0.4,
                 height=if(horizontal) 0.4 else 1.04,
                 x, plotAsp=0.1, plotID, turn,
                 add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)
lines_1d_graphics(loc.x=if(horizontal) c(-0.04, 1.04) else c(0.5, 0.5),
                  loc.y=if(horizontal) c(0.5, 0.5) else c(-0.04, 1.04),
                  type="l", x, horizontal=TRUE, plotAsp=0.05, plotID, turn,
                  add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)

points_2d_graphics(x, type="p", cex=0.4, plotID, turn,
                   add=FALSE, ann=FALSE, axes=FALSE,
                   xlim=c(0,1), ylim=c(0,1), ...)
density_2d_graphics(x, ngrids=25, nlevels=10, drawlabels=FALSE,
                    plotID, turn,
                    add=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)
axes_2d_graphics(x, length=0.1, eps=0.04, code=2, xpd=NA,
                 plotID, turn, add=FALSE, ann=FALSE, axes=FALSE,
                 xlim=c(0,1), ylim=c(0,1), ...)
label_2d_graphics(loc.x=1, loc.y=0, labels=NULL,
                  adj=c(1,0), cex=0.75, x, plotID, turn,
                  add=FALSE, ann=FALSE, axes=FALSE,
                  xlim=c(0,1), ylim=c(0,1), ...)
arrow_2d_graphics(loc=c(xlim[2], ylim[2])/2, length=0.2, angle=30,
                  x, plotID, turn,
                  add=FALSE, ann=FALSE, axes=FALSE,
                  xlim=c(0,1), ylim=c(0,1), ...)
rect_2d_graphics(loc.x=0.5, loc.y=0.5, width=1.08, height=1.08,
                 x, plotID, turn,
                 add=FALSE, ann=FALSE, axes=FALSE, xlim=c(0,1), ylim=c(0,1), ...)
}
\arguments{
  \item{x}{An \eqn{n}-\code{\link{vector}} (for 1d plots)
    or \eqn{(n,2)}-\code{\link{matrix}} (for 2d plots)
    containing the data. It is passed from \code{\link{zenplot}()}.}
  \item{width}{The width of the rugs.}
  \item{height}{The height of the rugs.}
  \item{horizontal}{A \code{\link{logical}} indicating whether the 1d plot
    is horizontal or vertical. It is passed
    from \code{\link{zenplot}()} (for 1d plots only).}
  \item{plotAsp}{The fraction (in [0,1]) of the shorter side divided by the
    longer side of the plot region. It is passed
    from \code{\link{zenplot}()} (for 1d plots only).}
  \item{plotID}{The plot identification passed on from \code{\link{zenplot}()}.
    This is a \code{\link{list}} containing the variable
    name(s), the column index(es) and the plotNo (among
    either all 1d or all 2d plots). It is passed
    from \code{\link{zenplot}()}.}
  \item{turn}{A \code{\link{character}} (in \code{"l"} (for \dQuote{left}),
    \code{"r"} (for \dQuote{right}), \code{"d"} (for \dQuote{down}),
    \code{"u"} (for \dQuote{up})) giving
    the turn as passed from \code{\link{zenplot}()}.}
  \item{add}{A \code{\link{logical}} indicating whether the current plot
    should be added to (or on top of) the previous one.}
  \item{ann}{A \code{\link{logical}} indicating whether the plot should
    be annotated.}
  \item{axes}{A \code{\link{logical}} indicating whether axes should be drawn.}
  \item{xlim,ylim}{The x and y limits of the plotting region.}
  \item{range}{The argument \code{range} of the underlying \code{\link{boxplot}()}
    (determines how far the plot whiskers extend out of the box). If
    \code{range=NULL}, this will be automatically determined depending
    on the sample size.}
  \item{breaks}{The break points for the histogram as passed to the
    underlying \code{\link{hist}()}. If \code{NULL}, the default is to
    use 20 equi-width bins covering the range of the data.}
  \item{col}{A vector of colors for the bars or bar components;
    see \code{\link{barplot}()}.}
  \item{loc}{The location of the center of the arrow.}
  \item{loc.x,loc.y}{x- and y-coordinates of the points combined by
    lines (for \code{lines_1d_graphics()}) or of the label
    (for \code{label_1d_graphics()}, \code{label_2d_graphics()}).}
  \item{type}{The plot type.}
  \item{density.args}{A \code{\link{list}()} of arguments for
    \code{\link{density}()}.}
  \item{offset}{A number in [0,0.5] determining the distance between the
    1d and 2d plots (for creating
    space between the two).}
  \item{method}{The type of density used.}
  \item{ngrids}{The number of grid points in each dimension (a scalar or
    an integer vector of length two).}
  \item{nlevels}{The number of contour levels to be drawn.}
  \item{drawlabels}{A \code{\link{logical}} indicating whether
    the contours should be labeled.}
  \item{labels}{The label to be used (with a useful default if
    \code{\link{NULL}}).}
  \item{srt}{The rotation of the label in degrees.}
  \item{cex}{The character expansion of the label or points.}
  \item{length}{The length of the arrow head.}
  \item{angle}{The angle between the shaft and one of the edges of the
    arrow head.}
  \item{eps}{Distance by which the axes are moved away from the plot region.}
  \item{code}{Integer code determining the kind of arrows to be drawn;
    see \code{\link{arrows}}.}
  \item{xpd}{A \code{\link{logical}} or \code{NA}, determining the
    region with respect to which clipping takes place; see \code{\link{par}()}.}
  \item{adj}{x (and optionally y) adjustment of the label.}
  \item{\dots}{Additional (mostly but not exclusively graphical)
    parameters passed to the underlying \pkg{graphics} functions.}
}
\value{
    \code{\link{invisible}()}.
}
\details{
    These functions based on the \R package \pkg{graphics} are provided
    as useful choices for the arguments \code{plot1d} and \code{plot2d}
    of \code{\link{zenplot}()}. See \code{\link{zenplot}()} how to use them,
    their source code for how to adjust them or how to write your own
    \code{plot1d} or \code{plot2d}. The main idea is that major arguments are
    passed to the underlying \pkg{graphics} functions as formal arguments and
    the ellipsis argument is used to pass all other (mostly graphical)
    paramaters. Hereby it is important to note that besides the
    ellipsis arguments (which are passed to both \code{plot1d} and
    \code{plot2d}), \code{\link{zenplot}()} passes further arguments to
    \code{plot1d} and \code{plot2d}. For \code{plot1d}, they are
    \code{x}, \code{horizontal}, \code{plotAsp}, \code{plotID},
    \code{turn} (as described above). For \code{plot2d}, they are
    \code{x}, \code{plotID}, \code{turn} (as described above).
}
\author{Marius Hofert and Wayne Oldford}
%% \references{
%%   Hofert, M., Oldford, W. (2015). Zigzag Expanded Navigation Plots.
%%   \emph{} \bold{}(), --.
%% }
\seealso{
  \code{\link{zenplot}()} for how to use these functions.
  \link[=rug_1d_grid]{plots_grid} for similar functions based on the \R
  package \pkg{grid}.
}
\examples{
## Implementation of 1d functions (for plot1d of zenplot())
rug_1d_graphics
points_1d_graphics
jitter_1d_graphics
density_1d_graphics
boxplot_1d_graphics
label_1d_graphics
arrow_1d_graphics
rect_1d_graphics
lines_1d_graphics

## Implementation of 2d functions (for plot2d of zenplot())
points_2d_graphics
density_2d_graphics
label_2d_graphics
arrow_2d_graphics
rect_2d_graphics
}
\keyword{dplot}