\name{zenplot}
\alias{unfold}
\alias{zenplot}
\title{Zenplot: Zigzag Expanded Navigation Plots}
\description{
  Construct and draw a zigzag expanded navigation plot for a graphical
  exploratory analysis of a path of variables.
}
\usage{
unfold(nVars, turns=NULL, n2dcol=5,
       method=c("tidy", "double.zigzag", "single.zigzag"),
       last1d=TRUE, width1d=1, width2d=10)
zenplot(x, scale=c("columnwise", "rowwise", "all", "pobs", "none"),
        turns=NULL, n2dcol=5,
        method=c("tidy", "double.zigzag", "single.zigzag"),
        pkg=c("graphics", "grid"),
        plot1d=c("rug", "points", "jitter", "density", "boxplot", "hist",
                 "label", "arrow", "rect", "lines"),
        plot2d=c("points", "density", "axes", "label", "arrow", "rect"),
        last1d=TRUE, width1d=NULL, width2d=NULL,
        par.list=list(mar=rep(0, 4), oma=rep(0.6, 4)),
        vp1d=NULL, vp2d=NULL, vp=viewport(width=unit(0.96, "npc"),
                                          height=unit(0.96, "npc")),
        draw=TRUE, ...)
}
\arguments{
  \item{x}{\eqn{(n,d)}-\code{\link{matrix}} (or
    \eqn{(n,d)}-\code{\link{data.frame}}) containing the data.}
  \item{scale}{A \code{\link{function}} applied to \code{x} for scaling
    to \eqn{[0,1]} (the default plot region) or a \code{\link{character}}
    string indicating the scaling method (in which case
    \code{\link{scale01}()} is called). The behavior for
    \code{scale="none"} is as follows: If \code{x} is in \eqn{[0,1]}, no
    scaling is done. If not, then \code{plot1d} and \code{plot2d} have
    to be \code{\link{function}}s which have to take care of proper
    scaling.}
  \item{nVars}{The number of variables. \code{zenplot()} calls
    \code{unfold()} with \code{nVars=ncol(x)}.}
  \item{turns}{A \code{\link{character}} vector of length two times the
    number of variables to be plotted minus 1 consisting of \code{"d"},
    \code{"u"}, \code{"r"} or \code{"l"}.}
  \item{n2dcol}{The number of columns containing bivariate (2d) plots.}
  \item{method}{The type of zigzag plot (a \code{\link{character}}).
    Available are:
    \describe{
      \item{\code{tidy}:}{A more tidied-up \code{double.zigzag}
	(slightly more compact placement of plots towards the end).}
      \item{\code{double.zigzag}:}{A zigzag plot in the form of a
	flipped \dQuote{S}. Along this path, the plots
	are placed in the form of an \dQuote{S} which is rotated
	counterclockwise by 90 degrees.}
      \item{\code{single.zigzag}:}{A zigzag plot in the form of a
	flipped \dQuote{S}.}
    }
    Note that \code{method} is ignored if \code{turns} are provided.}
  \item{plot1d}{A \code{\link{function}} in one variable returning a
    one-dimensional plot constructed with package \code{pkg}
    (typically a \code{\link{grob}} constructed with \code{pkg="grid"}).
    Alternatively, a \code{\link{character}} string indicating which
    of the provided default plots should be used. \code{plot1d} can also
    be \code{\link{NULL}} in which case no plot is constructed (for
    \code{pkg="grid"}, \code{\link{nullGrob}()} is returned).}
  \item{plot2d}{A \code{\link{function}} in two variables returning a
    two-dimensional plot constructed with package \code{pkg}
    (typically a \code{\link{grob}} constructed with \code{pkg="grid"}).
    Alternatively, a \code{\link{character}} string indicating which
    of the provided default plots should be used. \code{plot2d} can also
    be \code{\link{NULL}} in which case no plot is constructed (for
    \code{pkg="grid"}, \code{\link{nullGrob}()} is returned).}
  \item{pkg}{The R package used for plotting (depends on how the
    functions \code{plot1d} and \code{plot2d} were constructed;
    the user is responsible for choosing the appropriate package
    among the currently supported ones).}
  \item{last1d}{A \code{\link{logical}} indicating whether the last
    one-dimensional plot shall be included.}
  \item{draw}{A \code{\link{logical}} indicating whether a plot is
    created.}
  \item{width1d}{Graphical parameter giving the width of univariate (1d)
    plots. If \code{NULL} a default is chosen.}
  \item{width2d}{Graphical parameter giving the width of bivariate (2d)
    plots. If \code{NULL} a default is chosen.}
  \item{par.list}{Argument \code{\link{list}} to \code{\link{par}()} in
    case \code{pkg="graphics"}.}
  \item{vp1d,vp2d,vp}{The \code{\link{viewport}()}s passed to
    \code{plot1d}, \code{plot2d} and the overall frame
    \code{\link{grob}} (containing the whole zen plot), respectively.}
  \item{\dots}{Additional arguments passed to both \code{plot1d()} and
    \code{plot2d()}.}
}
\value{
  \code{unfold()} returns a \code{\link{list}} consisting of the path
  (itself a \code{\link{list}} containing turns (a \code{\link{character}}
  vector with elements in \dQuote{l}, \dQuote{r}, \dQuote{d}, \dQuote{u}),
  positions (a \code{\link{matrix}} of indices in
  the occupancy matrix) and the occupancy
  matrix itself (a \code{\link{matrix}} with elements in 0--4 where 0
  stands for \dQuote{not occupied} and 1--4 encode \dQuote{l},
  \dQuote{r}, \dQuote{d}, \dQuote{u})) and details about the layout
  (another \code{\link{list}}).

  \code{invisible} unless assigned to a variale in which case a
  \code{\link{list}} containing the path and layout (and the frame grob
  if \code{pkg="grid"}) is returned.
}
\author{Marius Hofert and Wayne Oldford}
%% \references{
%%   Hofert, M., Oldford, W. (2015). Zigzag Expanded Navigation Plots.
%%   \emph{} \bold{}(), --.
%% }
\seealso{
  All provided functions (of the form \code{*_1d_grid()} or
  \code{*_2d_grid()}) as arguments to \code{plot1d} or \code{plot2d};
  see \link[=rug_1d_grid]{plots_grid} or
  \link[=rug_1d_graphics]{plots_graphics}. \code{\link{scale01}()} for
  scaling the data.
}
\examples{
require(zenplots)


### How the zen layout works ###################################################

## Auxiliary function
correctly_fails <- function(x)
    is(tryCatch(x, error=function(e) e), "simpleError")

## Generate data
n <- 1000 # sample size
d <- 20 # dimension
set.seed(271) # set seed (for reproducibility)
x <- matrix(rnorm(n*d), ncol=d) # i.i.d. N(0,1) data

## Call zenplot() in various cases
## Zero 2d plots (1st plot on the path)
stopifnot(correctly_fails(zenplot(x[,1], n2dcol=0)))
zenplot(x[,1], n2dcol=1) # 1d
zenplot(x[,1], n2dcol=1, pkg="grid") # 1d; with grid

## One 2d plot (2nd/3rd plot on the path)
zenplot(x[,1:2], n2dcol=1, last1d=FALSE) # 1d, 2d
zenplot(x[,1:2], n2dcol=1) # 1d, 2d, 1d
zenplot(x[,1:2], n2dcol=1, last1d=FALSE, pkg="grid") # 1d, 2d; with grid
zenplot(x[,1:2], n2dcol=1, pkg="grid") # 1d, 2d, 1d; with grid

## Two 2d plots (4th/5th plot on the path)
zenplot(x[,1:3], n2dcol=2, last1d=FALSE) # 1d, 2d, 1d, 2d
stopifnot(correctly_fails(zenplot(x[,1:3], n2dcol=1))) # 1d, 2d, 1d, 2d, 1d
stopifnot(correctly_fails(zenplot(x[,1:3], n2dcol=1, method="double.zigzag"))) # 1d, 2d, 1d, 2d, 1d
stopifnot(correctly_fails(zenplot(x[,1:3], n2dcol=1, method="single.zigzag"))) # 1d, 2d, 1d, 2d, 1d
zenplot(x[,1:3], n2dcol=2) # 1d, 2d, 1d, 2d, 1d
zenplot(x[,1:3], n2dcol=2, method="double.zigzag") # 1d, 2d, 1d, 2d, 1d
zenplot(x[,1:3], n2dcol=2, method="single.zigzag") # 1d, 2d, 1d, 2d, 1d

## Three 2d plots (6th/7th plot on the path)
zenplot(x[,1:4], n2dcol=2, last1d=FALSE) # 1d, 2d, 1d, 2d, 1d, 2d
zenplot(x[,1:4], n2dcol=2) # 1d, 2d, 1d, 2d, 1d, 2d, 1d
zenplot(x[,1:4], n2dcol=2, method="double.zigzag") # 1d, 2d, 1d, 2d, 1d, 2d, 1d
zenplot(x[,1:4], n2dcol=2, method="single.zigzag") # 1d, 2d, 1d, 2d, 1d, 2d, 1d

## Four 2d plots (8th/9th plot on the path)
zenplot(x[,1:5], n2dcol=2, last1d=FALSE) # 1d, 2d, 1d, 2d, 1d, 2d, 1d, 2d
zenplot(x[,1:5], n2dcol=2) # 1d, 2d, 1d, 2d, 1d, 2d, 1d, 2d, 1d
zenplot(x[,1:5], n2dcol=3) # 1d, 2d, 1d, 2d, 1d, 2d, 1d, 2d, 1d
zenplot(x[,1:5], n2dcol=3, method="double.zigzag") # 1d, 2d, 1d, 2d, 1d, 2d, 1d, 2d, 1d
zenplot(x[,1:5], n2dcol=3, method="single.zigzag") # 1d, 2d, 1d, 2d, 1d, 2d, 1d, 2d, 1d

## Large sub-data sets
zenplot(x[,1:18], n2dcol=5)
zenplot(x[,1:18], n2dcol=5, method="double.zigzag")
zenplot(x[,1:18], n2dcol=5, method="single.zigzag")
zenplot(x[,1:19], n2dcol=5)

## The full data with different n2dcol
zenplot(x, n2dcol=4)
zenplot(x, n2dcol=5)
zenplot(x, n2dcol=6)
zenplot(x, n2dcol=7)
zenplot(x, n2dcol=7, pkg="grid") # with grid
zenplot(x, n2dcol=8)
zenplot(x, n2dcol=9)
zenplot(x, n2dcol=10)
zenplot(x, n2dcol=11)
zenplot(x, n2dcol=12)


### Providing your own turns ###################################################

turns <- c("l","d","d","r","r","d","d","r","r","u","u","r","r","u","u","l","l",
           "u","u","l","l","u","u","l","l","d","d", "l","l","d","d","l","l","d",
           "d","r","r","d","d")
zenplot(x, turns=turns)
zenplot(x, turns=turns, pkg="grid") # with grid


### Providing your own plot1d() or plot2d() ####################################

## Examples with predefined plot1d() or plot2d()
zenplot(x, plot1d="label", plot2d="rect")
zenplot(x, plot1d="label", plot2d="rect", pkg="grid")
zenplot(x, plot1d="arrow", plot2d="rect")
zenplot(x, plot1d="arrow", plot2d="rect", pkg="grid")
zenplot(x, plot1d="arrow", plot2d="arrow")
zenplot(x, plot1d="arrow", plot2d="arrow", pkg="grid")
zenplot(x, plot1d=NULL) # omitting the 1d plots
zenplot(x, plot1d=NULL, pkg="grid")
zenplot(x, plot2d=NULL) # omitting the 2d plots
zenplot(x, plot2d=NULL, pkg="grid")

## Iterate over all predefined plot1d()
methods <- eval(formals(zenplot)$plot1d)
for(p1d in methods) {
    zenplot(x, plot1d=p1d) # with graphics
    zenplot(x, plot1d=p1d, pkg="grid") # with grid
}

## 2-sided densities
zenplot(x, plot1d=function(...) density_1d_graphics(..., method="double"))
zenplot(x, plot1d=function(...)
        density_1d_grid(..., method="double"), pkg="grid")

## Colorized rugs as plot1d()
basecol <- c("royalblue3", "darkorange2", "maroon3")
palette <- colorRampPalette(basecol, space="Lab")
cols <- palette(d) # different color for each 1d plot
zenplot(x, plot1d=function(plotID, ...)
        rug_1d_graphics(col=cols[plotID$plotNo], ...))
require(grid) # for gTree() and gList()
zenplot(x, pkg="grid", # you are responsible for choosing the right pkg (cannot be tested!)
        plot1d=function(plotID, ...)
               rug_1d_grid(col=cols[plotID$plotNo], ...))

## Rectangles with labels as plot2d() (shows how to overlay plots)
## With graphics
zenplot(x, plot1d="arrow", plot2d=function(...){
            rect_2d_graphics(...); label_2d_graphics(..., add=TRUE)})
## With grid
zenplot(x, pkg="grid", plot1d="arrow", plot2d=function(x, plotID, ...)
    gTree(children=gList(rect_2d_grid(...),
                         label_2d_grid(x=x, plotID=plotID, ...))))

## Rectangles with labels outside the 2d plotting region as plot2d()
## With graphics
zenplot(x, plot1d="arrow", plot2d=function(...){
            rect_2d_graphics(...)
            label_2d_graphics(..., add=TRUE, xpd=NA, srt=90,
                              loc.x=1.08, loc.y=-0.04, adj=c(0,1), cex=0.56)})
## With grid
zenplot(x, pkg="grid", plot1d="arrow", plot2d=function(x, plotID, ...)
    gTree(children=gList(rect_2d_grid(...),
                         label_2d_grid(..., x=x, plotID=plotID,
                                       loc.x=1.03, loc.y=0,
                                       just=c("left", "top"),
                                       rot=90, cex=0.37))))
## Note: The above four plots show the variable names if x has column names

## An example showing how to modify graphical parameters of plot1d(), plot2d()
## With graphics
zenplot(x,
        plot1d=function(...) {
            lines_1d_graphics(col=basecol[1], lwd=1.4, ...)},
        plot2d=function(...) {
            rect_2d_graphics(..., border="grey50", lty=3)
            arrow_2d_graphics(..., add=TRUE)
            label_2d_graphics(..., col=basecol[3], add=TRUE)
})

## With grid
zenplot(x, pkg="grid",
        plot1d=function(horizontal, draw=FALSE, ...)
            lines_1d_grid(horizontal=horizontal,
                          col=basecol[1], lwd=1.4, draw=draw),
        plot2d=function(x, plotID, draw=FALSE, ...)
            gTree(children=gList(rect_2d_grid(...,
                                              col="grey50", lty=3, draw=draw),
                                 arrow_2d_grid(..., draw=draw),
                                 label_2d_grid(x=x, plotID=plotID,
                                               col=basecol[3], draw=draw))))

## A more sophisticated example

## 1) Build a random sample from a 17-dimensional grouped t copula
d. <- c(8, 5, 4) # sector dimensions
nu <- c(rep(10,d.[1]), rep(3,d.[2]), rep(0.5,d.[3])) # d.o.f. for each dimension
d <- length(nu) # total dimension
n <- 500 # sample size
set.seed(271)
Z <- matrix(rnorm(n*d), ncol=n) # (d,n)-matrix
Sigma <- matrix(0.5, nrow=d, ncol=d)
diag(Sigma) <- 1
L <- t(chol(Sigma)) # L: LL^T = Sigma
Y <- t(L \%*\% Z) # (n,d)-matrix containing n d-vectors following N(0,Sigma)
U. <- runif(n)
W <- sapply(nu, function(nu.) 1/qgamma(U., shape=nu./2, rate=nu./2)) # (n,d)-matrix
X <- sqrt(W) * Y # (n,d)-matrix
U <- sapply(1:d, function(j) pt(X[,j], df=nu[j])) # (n,d)-matrix

## 2) Plot the data with a pairs plot, colorizing the groups
colmat <- matrix("black", nrow=d, ncol=d)
colmat[1:d.[1], 1:d.[1]] <- basecol[1]
colmat[(d.[1]+1):(d.[1]+d.[2]), (d.[1]+1):(d.[1]+d.[2])] <- basecol[2]
colmat[(d.[1]+d.[2]+1):d, (d.[1]+d.[2]+1):d] <- basecol[3]
diag(colmat) <- NA # remove colors corresponding to diagonal entries (no plotting)
cols <- as.vector(colmat) # convert to a vector
cols <- cols[!is.na(cols)] # remove NA entries corresponding to diagonal
## Define panel function for colorizing the groups
count <- 0
mypanel <- function(x, y, ...) # panel function for colorizing groups
    { count <<- count+1; points(x, y, pch=".", col=cols[count]) }
## Plot
pairs(U, panel=mypanel, gap=0,
      labels=as.expression( sapply(1:d, function(j) bquote(italic(U[.(j)]))) ))

## 3) Zen plot of a random path through all pairs, colorizing the respective group
## Define our own points_2d_grid() for colorizing the groups
my_points_2d_grid <- function(x, plotID, draw=TRUE, ...) {
    var <- as.numeric(plotID$name) # pick out the names (variable numbers)
    col <- if(all(1 <= var & var <= d.[1])) { basecol[1] } else {
            if(all(d.[1]+1 <= var & var <= d.[1]+d.[2])) { basecol[2] } else {
               if(all(d.[1]+d.[2]+1 <= var & var <= d)) basecol[3] else "black"
            }
        }
    grid.points(x=x[,1], y=x[,2], pch=21, size=unit(0.02, "npc"),
                gp=gpar(col=col, ...), draw=draw)
}
## Plot a random permutation of columns via a zen plot
## Note: We set column labels here, as otherwise the labels can only
##       show *indices* of the variables to be plotted, i.e., the column
##       number in U[,ord], not the original column number in U (which
##       is what we want to see in order to see how our 'path' through
##       the pairs of variables looks like.
colnames(U) <- 1:d
set.seed(1)
(ord <- sample(1:d, size=d)) # path; 1:d would walk parallel to the secondary diagonal
zenplot(U[,ord], pkg="grid",
        plot1d=function(..., draw) arrow_1d_grid(...,
                        col="grey50", draw=FALSE),
        plot2d=function(x, plotID, draw=FALSE, ...)
            gTree(children=gList(
                  my_points_2d_grid(x=x, plotID, alpha=0.6, draw=draw),
                  rect_2d_grid(col="grey50", lty=3, draw=draw),
                  label_2d_grid(x=x, plotID=plotID, loc.x=1.02, loc.y=0,
                                just=c("left", "top"), rot=90, cex=0.4,
                                fontface="bold", draw=draw) )))
## => The points are colorized correctly (compare with the pairs plot)
}
\keyword{hplot}